/**
    @file ft51_i2c_master.h

    @brief     Functions to control the FT51 I2C Master.

    The FT51 has a single I2C master peripheral, compliant with
    the published I2C specification 
	http://www.nxp.com/documents/user_manual/UM10204.pdf
	The master is capable of addressing up to a maximum of 127 
	slave devices in 7-Bit mode.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */

#ifndef __FT51_I2C_MASTER_H__
#define __FT51_I2C_MASTER_H__

#include <stdint.h>
#include "ft51.h"

#define I2C_TIMER_HS_SELECT     0x80
#define I2C_TIMER_PERIOD_MASK   0x7F

/**
    @typedef I2C_master_status
    @brief TBD

    No details yet.
**/
typedef struct I2C_master_status I2C_master_status;

/**
    @struct I2C_master_status
    @brief Collection of I2C master status values.

**/
struct I2C_master_status
{
    /** Busy **/
    uint8_t busy:1;
    /** Error **/
    uint8_t error:1;
    /** Address ACK **/
    uint8_t address_ack:1;
    /** Data ACK **/
    uint8_t data_ack:1;
    /** Arbitration lost **/
    uint8_t arb_lost:1;
    /** Bus Idle **/
    uint8_t idle:1;
    /** Bus is busy, set and reset by START and STOP conditions **/
    uint8_t bus_busy:1;
    /** Unused **/
    uint8_t unused:1;
};

/**
    @brief   Call once, to initialise the master and reset it to a known state.

    It is not valid to call any other I2C Master functions before this one.
**/                                  
void I2C_master_initialise(void);

/**
    @brief   Read a specified number of bytes from an I2C device.

    Automatically adopts burst mode if the slave supports it, and
    more than one byte is to be read.

    @param[in]  addr             I2C address to read from.
    @param[in]  cmd              I2C command byte
    @param[in]  data             Buffer to read bytes into.
    @param[in]  number_to_read  Number of bytes to read.
**/
FT51_STATUS I2C_master_read (uint8_t addr, const uint8_t cmd,
        const uint8_t *data, const uint16_t number_to_read);

/**
    @brief   Write a specified number of bytes to an I2C device.

    Automatically adopts burst mode if the slave supports it, and
    more than one byte is to be written.

    @param[in]  addr             I2C address to write to.
    @param[in]  cmd              I2C command byte
    @param[in]  data             Buffer containing bytes to write.
    @param[in]  number_to_write  Number of bytes to read.
**/
void I2C_master_write(const uint8_t addr, const uint8_t cmd,
        const uint8_t *data, const uint16_t number_to_write);

/**
    @brief Set clock period for the I2C Master.

    When hs=0 The Master will automatically adopt the relevant I2C mode
    (Standard, Fast, Fast-Plus) depending on the timer parameter.
    When High-Speed is set with hs=1 The Master will adopt High-Speed
    mode. The maximum frequency is 3,400,000 Hz, but is limited to one
    tenth of the system clock frequency.
    The bit value I2C_TIMER_HS_SELECT is used to select hs value.
    The timer period value must be expressed in I2C_TIMER_PERIOD_MASK.
    If you change the system clock frequency, call this function
    again to ensure correct operation.

    @param[in] hs Non-zero to select High-Speed.
    @param[in] timer Timer period register value (max 127) and High-Speed select.
**/
void I2C_master_set_timer(uint8_t hs, uint8_t timer);

/**
    @brief   Determine I2C Master status.

    @return Returns the status byte of the I2C Master Status register
**/
uint8_t I2C_master_get_status(void);

#endif // __FT51_I2C_MASTER_H__