/**
    @file ft51_usb_tmc.c

    @brief USB Library API Test and Measurement Class functions.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "ft51.h"
#include "ft51_usb.h"
#include "ft51_usb_tmc.h"

char TMC_process_buffer[64];

/** Device Dependent Message callback function. **/
TMC_message_out_callback tmc_message_cmd_cb;
TMC_request_message_in_callback tmc_message_rsp_cb;


void tmc_initialise(TMC_ctx *ctx)
{
    tmc_message_cmd_cb = ctx->message_cmd_cb;
    tmc_message_rsp_cb = ctx->message_rsp_cb;
}

void tmc_class_req_capabilities(uint8_t devCaps, uint8_t interfaceCaps)
{
	get_capabilities_response_t bResp;

    memset (&bResp, 0, sizeof(get_capabilities_response_t));

    // Build the response packet
    bResp.USBTMC_status = USBTMC_STATUS_SUCCESS;

	bResp.USBTMC_Device_capabilities = devCaps;
	bResp.USBTMC_Interface_capabilities = interfaceCaps; //USBTMC_LISTEN_ONLY_MASK;

	// Required version of the TMC specification
	bResp.bcdUSBTMC = 0x0100;

	USB_transfer(USB_EP_0, USB_DIR_IN, (uint8_t*)&bResp, sizeof(bResp));
}

void tmc_class_init_clear( void )
{
	init_clear_response_t bResp;

	bResp.USBTMC_status = USBTMC_STATUS_SUCCESS;

	// Send the response packet before halting endpoint and flushing buffers
	USB_transfer(USB_EP_0, USB_DIR_IN, (uint8_t*)&bResp, sizeof(bResp));
}

void tmc_class_check_clear(void)
{
    check_clear_response_t bResp;

    bResp.USBTMC_status = USBTMC_STATUS_SUCCESS;
    bResp.bmClear = 0;

    USB_transfer(USB_EP_0, USB_DIR_IN, (uint8_t*)&bResp, sizeof(bResp));
}

void tmc_class_init_abort_bulk_out(uint8_t bTag, uint8_t ep_num, uint8_t ep_dir)
{
    abort_bulk_response_t bResp;

    USB_stall_endpoint(ep_num, ep_dir);

    // Build response packet
    bResp.USBTMC_status = USBTMC_STATUS_SUCCESS;
    bResp.bTag = bTag;       // bTag from received transaction

    // Forget about the last OUT command
    tmc_message_cmd_cb("");

    USB_transfer(USB_EP_0, USB_DIR_IN, (uint8_t*)&bResp, sizeof(bResp));
}

void tmc_class_check_abort_bulk_out(void)
{
    check_abort_bulk_out_status_response_t bResp;

    // Set to default value of zero
    memset(&bResp, 0, sizeof(check_abort_bulk_out_status_response_t));

    // Build response packet
    bResp.USBTMC_status = USBTMC_STATUS_SUCCESS;

    // Assume IN Buffer is not full
    //default: bResp.bmAbortBulkIn = 0;
    //default: bResp.reserved = 0;
    //default: NBYTES_RXD = 0;

    USB_transfer(USB_EP_0, USB_DIR_IN, (uint8_t*)&bResp, sizeof(bResp));
}

void tmc_class_init_abort_bulk_in(uint8_t bTag, uint8_t ep_num, uint8_t ep_dir)
{
    abort_bulk_response_t bResp;

    USB_stall_endpoint(ep_num, ep_dir);

    // Build response packet
    bResp.USBTMC_status = USBTMC_STATUS_SUCCESS;
    bResp.bTag = bTag;       // bTag from received transaction

    USB_transfer(USB_EP_0, USB_DIR_IN, (uint8_t*)&bResp, sizeof(bResp));
}

void tmc_class_check_abort_bulk_in(void)
{
    check_abort_bulk_in_status_response_t bResp;

    // Set to default value of zero
    memset(&bResp, 0, sizeof(check_abort_bulk_in_status_response_t));

    // Build response packet
    bResp.USBTMC_status = USBTMC_STATUS_SUCCESS;

	// Assume IN Buffer is not full
    //default: bResp.bmAbortBulkIn = 0;
    //default: bResp.reserved = 0;
    //default: NBYTES_TXD = 0;

	USB_transfer(USB_EP_0, USB_DIR_IN, (uint8_t*)&bResp, sizeof(bResp));
}


void tmc_process(void)
{
    uint8_t len = 0;
    uint16_t recvLen;

    recvLen = USB_transfer(USB_EP_1, USB_DIR_OUT, TMC_process_buffer, 64);

    if (recvLen)
    {
        if (TMC_process_buffer[0] == DEV_DEP_MSG_OUT)
        {
#ifdef TMC_VERIFY_PACKETS
            dev_dep_msg_out_t *msg_out = (dev_dep_msg_out_t *)recv;

            // Overall length is size of header and message.
            len = msg_out->TransferSize.b.transfer_size + sizeof(dev_dep_msg_out_t);

            // Messages are dword aligned - add padding length on.
            if (len % 4)
            {
                // Error - packet length not a multiple of 4
            }
#endif // TMC_VERIFY_PACKETS

            // Execute callback to application to handle message.
            tmc_message_cmd_cb(TMC_process_buffer + sizeof(dev_dep_msg_out_t));
        }
        else if (TMC_process_buffer[0] == REQUEST_DEV_DEP_MSG_IN)
        {
            dev_dep_msg_in_t *msg_in = (dev_dep_msg_in_t *)TMC_process_buffer;
#ifdef TMC_VERIFY_PACKETS
            uint16_t max_size;

            max_size = msg_in->TransferSize.b.lsb |
                    msg_in->TransferSize.b.mlsb << 8;
#endif // TMC_VERIFY_PACKETS

            // Set ID to Message IN
            msg_in->MsgId = DEV_DEP_MSG_IN;
            // This is the end of the message (single packet always)
            msg_in->bmTransferAttributes = TMC_TA_EOM_MASK;

            // Execute application callback to form response.
            len = tmc_message_rsp_cb(&TMC_process_buffer[sizeof(dev_dep_msg_in_t)]);

            // Messages are dword aligned - add padding length on.
            if (len % 4)
            {
                len += (4 - (len % 4));
            }

#ifdef TMC_VERIFY_PACKETS
            if (len > max_size)
            {
                len = max_size;
            }
#endif // TMC_VERIFY_PACKETS

            // Fill in message length.
            msg_in->TransferSize.b.lsb = len;
            msg_in->TransferSize.b.mlsb = 0;
            msg_in->TransferSize.w.msw = 0;

            // Length returned is size of header and returned message.
            len += sizeof(dev_dep_msg_in_t);
            USB_transfer(USB_EP_1, USB_DIR_IN, TMC_process_buffer, len);
        }
    }
}
