/**
    @file tmc_app.c

    @brief Source file for main functions of TMC Class demo on FT51.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */

/* INCLUDES ************************************************************************/

#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <stdio.h>

#include "ft51.h"
#include "ft51_general_config.h"
#include "ft51_interrupts.h"

#include "ft51_iomux.h"

#include "registers\ft51_registers.h"
#include "registers\ft51_iocell_registers.h"
#include "registers\ft51_iomux_registers.h"

#include "tmc_app_usb.h"
#include "adt7310.h"

#if LCD
#include "ft51_i2c_master.h"
#include "ft51_lcd.h"
#endif // LCD

/* CONSTANTS ***********************************************************************/

/* GLOBAL VARIABLES ****************************************************************/

/**
 @name ms_timer
 @brief Count down ms timer.
 */
volatile int16_t ms_timer = 0;

#if MEAS_FORCE

/**
 @name forceTimer
 @brief Count down ms timer for force. FORCE_TIMER is reset value to
 	 create a sampling timer for force.
 */
volatile int8_t forceTimer = 0;
#define FORCE_TIMER              50

/**
 @name force
 @brief Storage for current force reading from ADC sensor
*/
uint16_t force;

/**
 @name forceSample
 @brief Last sampled force reading from sensor.
*/
volatile uint16_t forceSample;

/**
 @name forceSampleReady
 @brief Sample of force reading from sensor is available.
*/
volatile uint8_t forceSampleReady;

#endif // MEAS_FORCE

#if MEAS_TEMP

/**
 @name tempTimer
 @brief Count down ms timer for temperature. TEMP_TIMER is reset
 	 value to create a sampling timer for the temperature.
 */
volatile int8_t tempTimer = 0;
#define TEMP_TIMER   100

/**
 @name temperature
 @brief Storage for current temperature reading
*/
uint16_t temperature = 0;

#endif // MEAS_TEMP

#if MEAS_HR

/**
 @name pulseTimer
 @brief Count down ms timer for heart rate. PULSE_TIMER is reset
 	 value to create a sampling timer for the heart rate.
 */
volatile int8_t pulseTimer = 0;
#define PULSE_TIMER      8 // Sample rate (in ms)

/**
 @name PULSE constants
 @brief Thresholds and constants for pulse detection.
*/
#define PULSE_SAMPLES   600 // Total number of samples held
#define PULSE_THRESHOLD  15 // Threshold value above which it is considered a possible heart beat
#define PULSE_IS_BEAT    20 // Number of threshold events close together which signifies a heart beat

/**
 @name pulseSample
 @brief Last sampled pulse reading from sensor.
*/
volatile uint16_t pulseSample;

/**
 @name pulseSampleReady
 @brief Sample of pulse reading from sensor is available.
*/
volatile uint8_t pulseSampleReady;

/**
 @name pulseSamples
 @brief Array of sample of pulse readings from sensor.
*/
static   uint8_t  pulseSamples[PULSE_SAMPLES];

/**
 @name pulseIndexer
 @brief Next index to save pulse value at (circular buffer) in pulseSamples
 	 array.
*/
static   uint16_t pulseIndexer = 0;
/**
 @name pulseCounterH/L
 @brief Used to count how many samples in a row are over the threshold in
 	 the pulseSamples array.
*/
static   uint8_t  pulseCounterHystRising;
static   uint8_t  pulseCounterHystFalling;

/**
 @name pulseAverage
 @brief Used to get a trigger value for pulses in the pulseSamples array.
*/
static   uint16_t  pulseAverage;

/**
 @name heartRate
 @brief Storage for current heart_rate reading
*/
uint16_t heartRate=0;

#endif // MEAS_HR

/* LOCAL VARIABLES *****************************************************************/

/* MACROS **************************************************************************/

/**
 @name Board Configuration
 @brief Mapping of pins on test board.
 */
//@{
// USB Hub Enabled - set to 1 for enabled and 0 for disabled
#ifndef PIN_REMOTE_WAKEUP
#define PIN_REMOTE_WAKEUP DIO_0
#endif // PIN_REMOTE_WAKEUP
//@}

#define LED_HR_ON() P3 &= ~((1 << 0))
#define LED_HR_OFF() P3 |= ((1 << 0))
#define LED_FORCE_ON() P3 &= ~((1 << 1))
#define LED_FORCE_OFF() P3 |= ((1 << 1))

/**
 @def MICROSECONDS_TO_TIMER_TICKS
 @brief Macro to calculate the number of timer ticks required to
 create a 1ms counter.
 Scale is clock frequency in MHz (assume 48) / clock-cycles per instruction.
 16-bit timer counts up to 65536, so subtract duration from 65536.
 */
#define MICROSECONDS_TO_TIMER_TICKS(a) (65536 - ((a) * (48 / 12)))

/* LOCAL FUNCTIONS / INLINES *******************************************************/

/**
 @brief      Invoked by ISR for Timer0 every millisecond
 @details    Each millisecond Timer0 interrupt handler will call
 this routine.
 When the detach counter is active (> zero) then we will decrement it.
 If we are in the DFUSTATE_APPDETACH state when it reaches zero then
 we will assume that the host has not send the USB reset required to
 put the device into DFU Mode and revert the state machine back to
 DFUSTATE_APPIDLE to wait for another DFU_DETACH request.
 We also make a timer for the keypress simulator.
 @param[in]  flags (not used).
 **/
void ms_timer_interrupt(const uint8_t flags)
{
    (void) flags; // Flags not currently used

    if (ms_timer)
    {
        ms_timer--;
    }

#if MEAS_HR
    if (pulseTimer)
    {
        pulseTimer--;
    }
#endif // MEAS_HR

#if MEAS_TEMP
    if (tempTimer)
    {
        tempTimer--;
    }
#endif // MEAS_TEMP

#if MEAS_FORCE
    if (forceTimer)
    {
        forceTimer--;
    }
#endif // MEAS_FORCE

#if DFU    
    // The DFU detach timer must be called once per millisecond
    dfu_timer();
#endif    

    // Reload the timer
    TH0 = MSB(MICROSECONDS_TO_TIMER_TICKS(1000));
    TL0 = LSB(MICROSECONDS_TO_TIMER_TICKS(1000));
}

/**
 @brief      Interrupt handler for ADC function
 @details    Retrieve sample values from analogue to digital
 	 	 	 converter and signals (via volatile variable) that
 	 	 	 a reading is for the force or heart rate sensors.
 @param[in]  flags (not used).
 **/
void adc_interrupt(const uint8_t flags)
{
    static uint8_t interrupt = 0;
    static uint8_t sample_l = 0x00;
    static uint8_t sample_h = 0x00;

    (void)flags;

#if MEAS_FORCE
    // Determine the interrupt source.
    IO_REG_INTERRUPTS_READ(AIO_INTERRUPTS_0_7, interrupt);
    // FSR Output
    if (interrupt & MASK_AIO_0_INT)
    {
        IO_REG_INTERRUPTS_READ(AIO_0_ADC_DATA_L_1, sample_l);
        IO_REG_INTERRUPTS_READ(AIO_0_ADC_DATA_U_1, sample_h);

        forceSample = ((sample_h << 8) | sample_l);
        forceSampleReady = TRUE;

        // Clear ADC interrupt register bit
        IO_REG_INTERRUPTS_WRITE(AIO_INTERRUPTS_0_7, MASK_AIO_0_INT);
    }
#endif

#if MEAS_HR
    // Determine the interrupt source.
    IO_REG_INTERRUPTS_READ(AIO_INTERRUPTS_8_15, interrupt);
    // Pulse Rate
    if (interrupt & MASK_AIO_10_INT)
    {
        IO_REG_INTERRUPTS_READ(AIO_10_ADC_DATA_L_1, sample_l);
        IO_REG_INTERRUPTS_READ(AIO_10_ADC_DATA_U_1, sample_h);

        pulseSample = ((sample_h << 8) | sample_l);
        pulseSampleReady = TRUE;

        // Clear ADC interrupt register bit
        IO_REG_INTERRUPTS_WRITE(AIO_INTERRUPTS_8_15, MASK_AIO_10_INT);
    }
#endif
}

/* FUNCTIONS ***********************************************************************/

/**
 @brief      Initialise Timer0 to fire every millisecond.
 @details    Associate the ms_timer_interrupt handler function with
 the Timer0 interrupt.
 **/
void ms_timer_initialise(void)
{
    // Register our own handler for interrupts from Timer 0
    interrupts_register(ms_timer_interrupt, interrupts_timer0);

    // Timer0 is controlled by TMOD bits 0 to 3, and TCON bits 4 to 5.
    TMOD &= 0xF0; // Clear Timer0 bits
    TMOD |= 0x01; // Put Timer0 in mode 1 (16 bit)
    // Set the count-up value so that it rolls over to 0 after 1 millisecond.
    TH0 = MSB(MICROSECONDS_TO_TIMER_TICKS(1000));
    TL0 = LSB(MICROSECONDS_TO_TIMER_TICKS(1000));
    TCON &= 0xCF; // Clear Timer0's Overflow and Run flags
    TCON |= 0x10; // Start Timer0 (set its Run flag)
}

/**
 @brief      Initialise I/O pins of FT51A
 @details    Sets up IOMUX to route signals to I/O pins.
 **/
void iomux(void)
{
#if MEAS_TEMP
    // SPI Master
    IOMUX_OUTPUT(DIO_6, IOMUX_OUT_SPI_MASTER_SS_N_0);
    IOMUX_INPUT(DIO_4, IOMUX_IN_SPI_MASTER_MISO);
    IOMUX_OUTPUT(DIO_3, IOMUX_OUT_SPI_MASTER_MOSI);
    IOMUX_OUTPUT(DIO_5, IOMUX_OUT_SPI_MASTER_SCLK);

    IOMUX_set_pad_config(DIO_6,
                        IOMUX_PAD_DRIVE_CURRENT_16MA,
                        IOMUX_PAD_TRIGGER_NORMAL,
                        IOMUX_PAD_SLEW_RATE_FAST,
                        IOMUX_PAD_PULL_NONE);
    IOMUX_set_pad_config(DIO_4,
                        IOMUX_PAD_DRIVE_CURRENT_16MA,
                        IOMUX_PAD_TRIGGER_NORMAL,
                        IOMUX_PAD_SLEW_RATE_FAST,
                        IOMUX_PAD_PULL_NONE);
    IOMUX_set_pad_config(DIO_3,
                        IOMUX_PAD_DRIVE_CURRENT_16MA,
                        IOMUX_PAD_TRIGGER_NORMAL,
                        IOMUX_PAD_SLEW_RATE_FAST,
                        IOMUX_PAD_PULL_NONE);
    IOMUX_set_pad_config(DIO_5,
                        IOMUX_PAD_DRIVE_CURRENT_16MA,
                        IOMUX_PAD_TRIGGER_NORMAL,
                        IOMUX_PAD_SLEW_RATE_FAST,
                        IOMUX_PAD_PULL_NONE);
#endif
#if LCD
    // I2C Master
    IOMUX_OUTPUT(DIO_8, IOMUX_OUT_PORT0_0);
    IOMUX_OUTPUT(DIO_13, IOMUX_OUT_I2C_MASTER_SCL);
    IOMUX_OUTPUT(DIO_14, IOMUX_OUT_I2C_MASTER_SDA);
    IOMUX_INPUT (DIO_14, IOMUX_IN_I2C_MASTER_SDA);

    IOMUX_set_pad_config(DIO_8,IOMUX_PAD_DRIVE_CURRENT_16MA,
                            IOMUX_PAD_TRIGGER_SCHMITT,
                            IOMUX_PAD_SLEW_RATE_FAST,
                            IOMUX_PAD_PULL_NONE);
    IOMUX_set_pad_config(DIO_13,IOMUX_PAD_DRIVE_CURRENT_8MA,
                            IOMUX_PAD_TRIGGER_SCHMITT,
                            IOMUX_PAD_SLEW_RATE_FAST,
                            IOMUX_PAD_PULL_NONE);
    IOMUX_set_pad_config(DIO_14,IOMUX_PAD_DRIVE_CURRENT_4MA,
                            IOMUX_PAD_TRIGGER_SCHMITT,
                            IOMUX_PAD_SLEW_RATE_FAST,
                            IOMUX_PAD_PULL_NONE);

#endif    

    // For USB compliance only...
    IOMUX_INPUT (PIN_REMOTE_WAKEUP, IOMUX_IN_MISC_WAKEUP_PIN);
    IOMUX_set_pad_config(PIN_REMOTE_WAKEUP,
                        IOMUX_PAD_DRIVE_CURRENT_4MA,
                        IOMUX_PAD_TRIGGER_NORMAL,
                        IOMUX_PAD_SLEW_RATE_FAST,
                        IOMUX_PAD_PULL_UP);
    IO_REG_GENERAL_WRITE(PIN_CONFIG, MASK_VBUS_DETECT_MODE);

    // EVM Board LEDs
    IOMUX_OUTPUT(AIO_11, IOMUX_OUT_PORT3_0); /* Heartbeat LED */
    IOMUX_OUTPUT(DIO_1, IOMUX_OUT_PORT3_1); /* Force LED */
}


/**
 @brief      Initialise ADC.
 @details    Setup the analogue to digital converter to sample
 	 	 	 inputs from the force and heart rate sensors.
 **/
#if defined(MEAS_FORCE) || defined(MEAS_HR)
void adc_setup (void) {
    // Enable Force and Pulse ADCs.
    // Enable AIO Cell Controller
    IO_REG_GENERAL_WRITE(AIO_CONTROL, MASK_AIO_DEV_EN);
#if MEAS_FORCE
    // Select mode. Setup AIO_0 to ADC
    IO_REG_GENERAL_WRITE(AIO_MODE_0, MASK_AIO_0_MODE_1);
    // Enable AIO_0 interrupt
    IO_REG_GENERAL_WRITE(AIO_INTERRUPT_ENABLES_0_7, MASK_AIO_0_IEN);
#endif
#if MEAS_HR
    // Select mode. Setup AIO_10 to ADC
    IO_REG_GENERAL_WRITE(AIO_MODE_2, MASK_AIO_10_MODE_1);
    // Enable AIO_8 interrupt
    IO_REG_GENERAL_WRITE(AIO_INTERRUPT_ENABLES_8_15, MASK_AIO_10_IEN);
#endif
    // Register for ADC interrupts
    interrupts_register(adc_interrupt, interrupts_IO_cell);
}
#endif /*defined(MEAS_FORCE) || defined(MEAS_HR)*/

/**
 @brief      main function.
 @details    Modify any configuration parameters which are updatable
 and call initialisation routines.
 Finally process USB requests as they arrive (for ever).
 **/
void main(void)
{
	static int j = 0;

    // Initialise FT51A
    device_initialise();

    // Setup the IOMUX to route signals to pins
    iomux();

    // Initialise USB function
    usb_setup();

#if defined(MEAS_HR) || defined(MEAS_FORCE)
    // Setup ADC for force measurement
    adc_setup();
#endif

    // Initialise the timer to provide a millisecond countdown timer
    ms_timer_initialise();

#if LCD
    // Configure the I2C Master
    I2C_master_initialise();

    // Reset the LCD display
    P0 = 0xf8; // Set reset line of LCD high for 5ms
    ms_timer = 5;
    while (ms_timer);
    P0 = 0xf9; // Clear reset line of LCD

    // Wait 40ms after reset
    ms_timer = 40;
    while (ms_timer);

    // Initialise the display
    lcd_initialise();
    // Clear the display
    lcd_clear();

    // Wait for 2ms to clear to complete
    ms_timer = 2;
    while (ms_timer);

    // Write welcome message to LCD
    lcd_position(0,0);
    lcd_display("AN_347 TMC_APP ", 15);
#endif

    //LED_HR_OFF();
    //LED_FORCE_OFF();
    P3 |= ((1 << 0) | (1 << 1));

#if MEAS_TEMP
    temperature_setup();
    // Wait for 2ms to setup to complete
    ms_timer = 2;
    while (ms_timer);

    temperature_reset();
    //temperature_configure();
#endif

#if MEAS_FORCE
    // Initiate ADC Sample on AIO_0
    IO_REG_GENERAL_WRITE(AIO_SAMPLE_0, MASK_AIO_0_SAMPLE);
#endif

#if MEAS_HR
    for(j = 0; j < PULSE_SAMPLES; ++j) pulseSamples[j] = PULSE_THRESHOLD;
    // Initiate ADC Sample on AIO_8
    IO_REG_GENERAL_WRITE(AIO_SAMPLE_1, MASK_AIO_10_SAMPLE);
#endif

    // Endless loop.
    while (1)
    {
        USB_process();
#if TMC
#if DFU
        if((USB_get_state() == CONFIGURED) && dfu_is_runtime())
#else // DFU
        if(USB_get_state() == CONFIGURED)
#endif // DFU
        {
            tmc_process();
        }
#endif // TMC


#if MEAS_TEMP
    	if (tempTimer == 0)
    	{
        	// Read the temperature from the SPI Master
        	temperature = temperature_read();
        	// Send a reset to sensor to ensure it's SPI slave stays
        	// synchronised with out SPI master.  This also initiates a pulseSampleTrig!
            // temperature_reset();

        	tempTimer = TEMP_TIMER;
    	}
#endif

#if MEAS_FORCE
    	if (forceTimer == 0)
        {
            IO_REG_INTERRUPTS_WRITE(AIO_SAMPLE_0, MASK_AIO_0_SAMPLE);

            forceTimer = FORCE_TIMER;
        }

        if (forceSampleReady)
        {
        	forceSampleReady = FALSE;

			if (forceSample > 900)
			{
				forceSample = 900;
			}

			force = forceSample;
        }

        if (force > 10)
		{
			LED_FORCE_ON();
		}
		else
		{
			LED_FORCE_OFF();
		}
#endif

#if MEAS_HR
    	if (pulseTimer == 0)
        {
            IO_REG_INTERRUPTS_WRITE(AIO_SAMPLE_1, MASK_AIO_10_SAMPLE);

            pulseTimer = PULSE_TIMER;
        }

    	// ------------------------------------
        // Get HeartRate sample
        // ------------------------------------
    	if (pulseSampleReady)
    	{
    		pulseSampleReady = FALSE;
			if (pulseSample > 0xff)
				pulseSample = 0xff;
			pulseSamples[pulseIndexer] = pulseSample;

			if (pulseSample > PULSE_THRESHOLD)
				LED_HR_ON();
			else
				LED_HR_OFF();

			// Count the total number of beats (for the entire sample period) ...
			heartRate = 0;
			pulseCounterHystRising = pulseCounterHystFalling = 0;
			for (j = 0; j < PULSE_SAMPLES; j++)
			{
				if (pulseSamples[j] > PULSE_THRESHOLD)
				{
					// Look for PULSE_IS_BEAT samples in a row to be above the
					// threshold.
					pulseCounterHystRising++;
					if (pulseCounterHystRising == PULSE_IS_BEAT)
					{
						pulseCounterHystFalling = PULSE_IS_BEAT;
						heartRate++;
					}
				}
				else
				{
					if (pulseCounterHystFalling)
					{
						pulseCounterHystFalling--;
						pulseCounterHystRising = 0;
					}
				}
			}
			// Correct for sample array size. i.e. 600 samples at 25ms intervals
			// will be 15 seconds of data. So multiply by 60/15 = 4 times to get
			// pulse per minute.
			heartRate *= (60000 / (PULSE_SAMPLES * PULSE_TIMER));

			pulseIndexer = (++pulseIndexer) % PULSE_SAMPLES;
    	}
#endif // MEAS_HR

#if LCD
	    if (ms_timer == 0)
    	{
    	    static char str[24];
    	    static char spinner = ' ';

    	    // Update LCD every 500 ms.
    	    ms_timer = 500;

    	    // Toggle a character to show activity.
			if (spinner == ' ') spinner = '.';
			else                spinner = ' ';

			lcd_position(1,0);
			j = sprintf(str, "%c ", spinner);
			lcd_display(str, j);

#if MEAS_FORCE
			j = sprintf(str, "%4uN " ,forceSample);
			lcd_display(str, j);
#endif
#if MEAS_TEMP
			j = sprintf(str, "%2u.%02uC " ,temperature/100, temperature%100);
			lcd_display(str, j);
#endif
#if MEAS_HR
			j = sprintf(str, "%3ubpm" ,heartRate);
			lcd_display(str, j);
#endif
    	}
#endif

    } // of while (1)
}
